# ==================================================================
# Plugins::SlideShow - Auto Generated Install Module
#
#   Plugins::SlideShow
#   Author  : Gossamer Threads Inc.
#   Version : 1.0
#   Updated : Tue Nov 27 17:18:24 2001
#
# ==================================================================
#

package Plugins::SlideShow;
# ==================================================================
    use strict;
    use vars  qw/$VERSION $DEBUG $NAME $META/;
    use GT::Base;
    use GT::Plugins qw/STOP CONTINUE/;
    use Links qw/$CFG $IN $DB/;
    
    $VERSION = '1.20070309';
    $DEBUG   = 0;
    $NAME    = 'SlideShow';
# Inhert from base class for debug and error methods
    @Plugins::SlideShow::ISA = qw(GT::Base);
    
    $META = {
    'prog_ver' => '2.1.1',
    'url' => 'http://www.gossamer-threads.com/',
    'license' => 'Commercial',
    'description' => 'This uses the SlideShow libraries and packages to allow you to run a photo gallery. This plugin automatically creates thumbnails of uploaded pictures and allows you to view them in a special frame.',
    'author' => 'Gossamer Threads Inc.',
    'version' => '1.20070309'
};


sub pre_install {
# -------------------------------------------------------------------
# This function displays an HTML formatted message that will display
# to the user any instructions/information before they install
# the plugin.
#
    my $inst_msg = '';

# test if important directories are writeable
    my @errs     = ();
    my $admin_dir = $CFG->{admin_root_path};
    foreach my $dir ( "$admin_dir/Plugins", "$admin_dir/..", "$admin_dir/templates/admin/help", "$admin_dir/templates/$CFG->{build_default_tpl}" ) {
# , "$admin_dir/templates/admin/help"
        if ( !-d $dir ) {
            push @errs, "`$dir' must be a valid directory!";
        }

        elsif ( !-w $dir ) {
            push @errs, "`$dir' must be a writeable directory!";
        }
    }

    my $build_images_path = $CFG->{build_root_path} . '/images';

# test to find out if Image::Magic has been installed
    $inst_msg    .= "Looking for Image::Magic - ";
    eval {
        require Image::Magick;
    };
    if ( $@ ) {
        $inst_msg .= qq~
        <font color=red>Load Unsuccessful</font> ($@)

        <p>
        This plugin requires the perl module Image::Magick. This module can be found on CPAN. Please note this perl module relies on a compiled library also Imagemagick which can be acquired at this location: <a href="http://www.imagemagick.org/">http://www.imagemagick.org/</a>

        <p>
        Please install the library before proceeding. This module will not work without this library and will cause your Links SQL installation to malfunction.

        <p>
        However, although highly unrecommended and may cause your entire installation of Links SQL to fail, you may choose to ignore this error and proceed to install.
        ~;
    } else {
        $inst_msg .= qq~<font color=green>Good</font>\n~;
    }

# Ensure that the plugin can write to all the directories in question
    my $err_str  = @errs ? qq~Based on a preliminary examination of your system the following issues will hamper installation:<br><br><font color="red"><ul><li>~
                    . join( "<li>", @errs )
                    . qq~</ul></font>You may chose to ignore these warnings and force installation.~ : '';



    $inst_msg .= qq~

    <p>
    To commence the installation this plugin must know the location of your images directory. Please put the path that &lt;%build_images_url%&gt; references. A guess has been made already, however it may be incorrect.

    <p>
    Build Images Path: <input type="text" name="build_images_path" value="$build_images_path">

    <p>
    This plugin will also create the column "SlideShowCache" in your Links table. This will allow the SlideShow plugin to cache information such as width/height information for each image as well as the URL location in a format that can be recalled quickly.

    <p>
    $err_str
    ~;

    return $inst_msg;
}

sub pre_uninstall {
# -------------------------------------------------------------------
# This function displays an HTML formatted message that will display
# to the user any instructions/information before they remove the
# plugin.
#
    my $uninst_msg = '';

    return $uninst_msg;
}

sub install {
# -------------------------------------------------------------------
# This function does the actual installation. It's first argument is
# a plugin manager which you can use to register hooks, install files,
# add menu options, etc. The second argument is a GT::Tar object which
# you can use to access any files in your plugin module.
#
# You should return an HTML formatted string that will be displayed 
# to the user.
#
# If there is an error, return undef, and set the error message in
# $Plugins::SlideShow::error
#
    my ($mgr, $tar) = @_;

    my $build_images_path = $IN->param('build_images_path') or return Plugins::SlideShow->error("Build Images Path has not been defined. Plugin must have this path to install the images come with this installation", 'WARN');
    ( -d $build_images_path and -w $build_images_path ) or return Plugins::SlideShow->error("'$build_images_path' does not exist or is not writeable", 'WARN');

    $mgr->install_hooks   ( 'SlideShow', [ ['add_link', 'PRE', 'Plugins::SlideShow::add_link', 'FIRST'] ]);
    $mgr->install_hooks   ( 'SlideShow', [ ['modify_link', 'PRE', 'Plugins::SlideShow::modify_link', 'FIRST'] ]);
    $mgr->install_hooks   ( 'SlideShow', [ ['user_modify_link', 'PRE', 'Plugins::SlideShow::check_input', 'FIRST'] ]);

# options
    $mgr->install_options ( 'SlideShow', [ ['max_upload_constraints', '2000x2000', 'Maximum size allowed for the uploaded image.'] ] );
    $mgr->install_options ( 'SlideShow', [ ['max_upload_size', '1000000', 'Maximum size in bytes for the uploaded image.'] ] );
    $mgr->install_options ( 'SlideShow', [ ['thumbnail_constraints', '100x100', 'Maximum width and height for thumbnail'] ] );
    $mgr->install_options ( 'SlideShow', [ ['medium_constraints', '500x500', 'Maximum width and height for medium sized image.'] ] );
    $mgr->install_options ( 'SlideShow', [ ['large_constraints', '', 'Maximum width and height for large image.'] ] );
    $mgr->install_options ( 'SlideShow', [ ['largest_constraints', '', 'Maximum width and height for largest image.'] ] );
    $mgr->install_options ( 'SlideShow', [ ['image_quality', '75', 'Used for JPGs and other lossy formats. Enter the percentage quality desired.'] ] );
    $mgr->install_options ( 'SlideShow', [ ['watermark_path', '', 'Path to the watermark file. Should be a small black and white image.'] ] );
    $mgr->install_options ( 'SlideShow', [ ['image_cols', '', 'Name of image columns, separate with commas'] ] );
    $mgr->install_options ( 'SlideShow', [ ['seq_image_cols', '', 'For the next/previous pager on the showpicture.cgi. The list of columns that can be paged through.'] ] );
    $mgr->install_options ( 'SlideShow', [ ['temp_dir', '/tmp', 'Temporary directory for image work'] ] );
    $mgr->install_options ( 'SlideShow', [ ['image_url_path', '', 'URL to the uploaded images. This is the http:// prefixed URL to the directory that will receive all your images.'] ] );

# menus
    $mgr->install_menu ( 'SlideShow', [ ['Help', 'admin.cgi?do=help&topic=SlideShow/help.html" target="_blank'] ] );
    $mgr->install_menu ( 'SlideShow', [ ['Field Management', 'admin.cgi?do=plugin&plugin=SlideShow&func=field_management'] ] );
    $mgr->install_menu ( 'SlideShow', [ ['Edit', 'admin.cgi?do=page&page=plugin_manager.html&plugin_man_do=edit_installed&plugin_name=SlideShow'] ] );
    $mgr->install_menu ( 'SlideShow', [ ['Resize', 'nph-imageresize.cgi'] ] );

# The following section will unarchive attached files into the 
# proper location.
    my $file;

# Copying showpictures.cgi to $CFG->{admin_root_path}/.. directory.
    $file = $tar->get_file ('showpicture.cgi');
    $file->name("$CFG->{admin_root_path}/../showpicture.cgi");
    $file->mode( 0755 );
    $file->write or return Plugins::SlideShow->error("Unable to extract file: '$CFG->{admin_root_path}/../showpicture.cgi' ($GT::Tar::error)", 'WARN');

# Copying nph-imagesize.cgi to $CFG->{admin_root_path}/.. directory.
    $file = $tar->get_file ('nph-imageresize.cgi');
    $file->name("$CFG->{admin_root_path}/nph-imageresize.cgi");
    $file->mode( 0755 );
    $file->write or return Plugins::SlideShow->error("Unable to extract file: '$CFG->{admin_root_path}/nph-imageresize.cgi' ($GT::Tar::error)", 'WARN');

# Copying pictureframe.html to $CFG->{admin_root_path}/.. directory.
    $file = $tar->get_file ('pictureframe.html');
    $file->name("$CFG->{admin_root_path}/templates/$CFG->{build_default_tpl}/pictureframe.html");
    $file->write or return Plugins::SlideShow->error("Unable to extract file: '$CFG->{admin_root_path}/templates/$CFG->{build_default_tpl}/pictureframe.html' ($GT::Tar::error)", 'WARN');

# Make the help directory for this plugin
    -d "$CFG->{admin_root_path}/templates/admin/help/SlideShow" or mkdir "$CFG->{admin_root_path}/templates/admin/help/SlideShow", 0777 or return Plugins::SlideShow->error("Unable to create directory '$CFG->{admin_root_path}/templates/admin/help' due to ($!)", 'WARN');

# Copying help.html to $CFG->{admin_root_path}/templates/admin/help. directory.
    $file = $tar->get_file ('help.html');
    $file->name("$CFG->{admin_root_path}/templates/admin/help/SlideShow/help.html");
    $file->write or return Plugins::SlideShow->error("Unable to extract file: '$CFG->{admin_root_path}/templates/admin/help/SlideShow/help.html' ($GT::Tar::error)", 'WARN');

# Copying screenshot.gif to $CFG->{admin_root_path}/templates/admin/help. directory.
    $file = $tar->get_file ('screenshot.gif');
    $file->name("$CFG->{admin_root_path}/templates/admin/help/SlideShow/screenshot.gif");
    $file->write or return Plugins::SlideShow->error("Unable to extract file: '$CFG->{admin_root_path}/templates/admin/help/SlideShow/screenshot.gif' ($GT::Tar::error)", 'WARN');

# Create the SlideShow column in the Links table
    my $ltbl = $DB->table( "Links" );
    unless ( $ltbl->cols->{ "SlideShowCache" } ) {
        my $e = $DB->editor( "Links" );
        $e->add_col( 
            "SlideShowCache" ,
            { 
                type => 'text',
                form_type => 'HIDDEN',
            }
        ) or return Plugins::SlideShow->error( "Could not add 'SlideShowCache' column because: $GT::SQL::error" );

        $CFG->{add_system_fields}{SlideShowCache} ||= '{}';
        $CFG->save or Plugins::SlideShow->error( "Couldn't update ConfigData with add_system_fields/SlideShowCache = '{}'" );
    }

# copy all the images for the picture frame to the default image directory
    foreach my $fname ( qw(
                            arrow.gif camera.gif dot.gif end.gif 
                            line_bg2.gif line_bg.gif next.gif 
                            previous.gif start.gif tab_corner2.gif 
                            tab_corner.gif title2.gif title3.gif 
                            title4.gif title5.gif title.gif 
                          ) ) {

    $file = $tar->get_file ( $fname );
    $file->name("$build_images_path/$fname");
    $file->write or return Plugins::SlideShow->error("Unable to extract file: '$build_images_path/$fname' ($GT::Tar::error)", 'WARN');


    }
 
    return "The plugin has been successfully installed!";
}

sub uninstall {
# -------------------------------------------------------------------
# This function removes the plugin. It's first argument is
# also a plugin manager which you can use to register hooks, install files,
# add menu options, etc. You should return an HTML formatted string
# that will be displayed to the user.
#
# If there is an error, return undef, and set the error message in
# $Plugins::SlideShow::error
#
    my $mgr = shift;
    
    $mgr->uninstall_hooks ( 'SlideShow', [ ['add_link', 'PRE', 'Plugins::SlideShow::add_link', 'FIRST'] ]);
    $mgr->uninstall_hooks ( 'SlideShow', [ ['modify_link', 'PRE', 'Plugins::SlideShow::modify_link', 'FIRST'] ]);
    
    return "The plugin has been successfully removed!";
}

1;

