# ==================================================================
# Plugins::ConvertVideo - Auto Generated Install Module
#
#   Plugins::ConvertVideo
#   Author  : Gossamer Threads Inc.
#   Version : 1.1
#   Updated : Wed Feb  10 13:42:33 2010
#
# ==================================================================
#

package Plugins::ConvertVideo;
# ==================================================================
use strict;
use vars  qw/$VERSION $DEBUG $NAME $META/;
use GT::Base;
use GT::Plugins qw/STOP CONTINUE/;
use Links qw/:objects/;

$VERSION = '1.1';
$DEBUG   = 0;
$NAME    = 'ConvertVideo';
# Inhert from base class for debug and error methods
@Plugins::ConvertVideo::ISA = qw(GT::Base);

$META = {
    'author' => 'Gossamer Threads Inc.',
    'description' => 'Convert mpeg video to flash file.',
    'license' => 'Other',
    'prog_ver' => '3.x',
    'url' => 'http://www.gossamer-threads.com',
    'version' => '1.1'
};


sub pre_install {
# -----------------------------------------------------------------------------
# This function displays an HTML formatted message that will display any
# instructions/information to the user before they install the plugin.
#
    my $inst_msg = qq~
Please enter the path where you want to store the FlowPlayer files (swf, javascript): 
<input type="text" name="flowplayer_path" value="" size="50"><br />
Enter the URL to the FlowPlayer files:
<input type="text" name="flowplayer_url" value="" size="50"><br />
	~;

    return $inst_msg;
}

sub pre_uninstall {
# -----------------------------------------------------------------------------
# This function displays an HTML formatted message that will display any
# instructions/information to the user before they remove the plugin.
#
    my $uninst_msg = '';

    return $uninst_msg;
}

sub install {
# -----------------------------------------------------------------------------
# This function does the actual installation.  Its first argument is a plugin
# manager which you can use to register hooks, install files, add menu options,
# etc.  The second argument is a GT::Tar object which you can use to access any
# files in your plugin module.
#
# You should return an HTML formatted string that will be displayed to the
# user.
#
# If there is an error, return undef, and set the error message in
# $Plugins::ConvertVideo::error
#
    my ($mgr, $tar) = @_;

	my $flowplayer_url = $IN->param('flowplayer_url') || $CFG->{build_root_url};
 	my $flowplayer_path = $IN->param('flowplayer_path') || $CFG->{build_root_path};
   
    $mgr->install_hooks('ConvertVideo', [['validate_link', 'PRE', 'Plugins::ConvertVideo::validate_link_pre', '']]);
    $mgr->install_hooks('ConvertVideo', [['modify_link', 'PRE', 'Plugins::ConvertVideo::modify_link_pre', '']]);
    $mgr->install_hooks('ConvertVideo', [['modify_link', 'POST', 'Plugins::ConvertVideo::modify_link_post', '']]);
    $mgr->install_hooks('ConvertVideo', [['add_link', 'POST', 'Plugins::ConvertVideo::add_link_post', '']]);
    $mgr->install_hooks('ConvertVideo', [['form_link', 'PRE', 'Plugins::ConvertVideo::pre_form_link', '']]);
    $mgr->install_hooks('ConvertVideo', [['form_link', 'POST', 'Plugins::ConvertVideo::post_form_link', '']]);
    $mgr->install_options('ConvertVideo', [['video_file_field', '', 'Field name of the video file field.', 'TEXT', [], [], '']]);
    $mgr->install_options('ConvertVideo', [['flash_file_field', '', 'Field name of the flash file field.', 'TEXT', [], [], '']]);
    $mgr->install_options('ConvertVideo', [['image_file_field', '', 'Field name of the preview image file field.', 'TEXT', [], [], '']]);
    $mgr->install_options('ConvertVideo', [['thumbnail_file_field', '', 'Field name of the thumbnail file field.', 'TEXT', [], [], '']]);
    $mgr->install_options('ConvertVideo', [['flash_dimension', '320x240', 'The size of the flash movie (e.g. 450x370)', 'TEXT', [], [], '']]);
    $mgr->install_options('ConvertVideo', [['flash_quality', '6', 'quality scale (1 [best] - 31 [worst])', 'TEXT', [], [], '']]);
    $mgr->install_options('ConvertVideo', [['thumbnail_size', '80x80', 'The size of the thumbnail.', 'TEXT', [], [], '']]);
    $mgr->install_options('ConvertVideo', [['watermark_file', '', 'The path to the watermark image.', 'TEXT', [], [], '']]);
    $mgr->install_options('ConvertVideo', [['video_url', '', 'The url where you store the video files.', 'TEXT', [], [], '']]);
    $mgr->install_options('ConvertVideo', [['flowplayer_url', $flowplayer_url, 'The url path where you store the flowplayer swf and javascript files.', 'TEXT', [], [], '']]);

    my $output;
#---------- Support Files -----------------
	my $files = $tar->files;

	foreach my $file (@$files) {
		my $name = $file->name;
		if ($name =~ /\.cgi$/) {
			$name =~ s/.*\///;
			my $file_fpath = $CFG->{admin_root_path} . "/" . $name;
			my $body = $file->body_as_string;
			$body =~ s,^(#!)[^\n]+,$1$CFG->{path_to_perl},;
			$body =~ s/use lib 'admin';/use lib '$CFG->{admin_root_path}';/;
			$body =~ s/init\('admin'\);/init('$CFG->{admin_root_path}');/;
			$file->body($body);
			$file->name( $file_fpath  );
			$file->mode(0755);
			$file->write;
		}
		elsif ($name=~/\.swf$/) {
			$name=~s/.*\///;
			$file->name("$flowplayer_path/$name");
			$file->mode(0644);
			$file->write;
		}
		elsif ($name=~/^fullscreen/ || $name=~/\.js$/) {
			$name=~s/.*\///;
			my $body = $file->body_as_string;
			$body =~ s/\$flowplayerurl/$flowplayer_url/g;
			$file->body($body);
			$file->name("$flowplayer_path/$name");
			$file->mode(0666);
			$file->write;
		}
		elsif ($name=~/.html$/) {
			$name=~s/.*\///;
			unless ((-e "$CFG->{admin_root_path}/templates/$CFG->{build_default_tpl}/$name") && ($IN->param('overwrite') eq 'No')) {
				my $body = $file->body_as_string;
				$body =~ s/\$flowplayerurl/$flowplayer_url/g;
				$file->body($body);
				$file->name("$CFG->{admin_root_path}/templates/$CFG->{build_default_tpl}/$name");
				$file->mode(0666);
				$file->write;
			}
		}
	}

    $output =~ s/\n/<br>/g;

    return $output . "The plugin has been successfully installed!";
}

sub uninstall {
# -----------------------------------------------------------------------------
# This function removes the plugin.  Its first argument is also a plugin
# manager which you can use to register hooks, install files, add menu options,
# etc. You should return an HTML formatted string that will be displayed to the
# user.
#
# If there is an error, return undef, and set the error message in
# $Plugins::ConvertVideo::error
#
    my $mgr = shift;
    
    return "The plugin has been successfully removed!";
}

1;
