#!/usr/bin/perl -w
# ==================================================================
#
# If you see this after you have uploaded this file to your 
# webserver your server is not recognising this file as a cgi 
# properly. Please contact your server administrator to 
# find out how to have this test script execute properly
#
# ==================================================================
# Links SQL - enhanced directory management system
#
#   Website  : http://gossamer-threads.com/
#   Support  : http://gossamer-threads.com/scripts/support/
#   CVS Info : 087,068,085,090,083 
#   Revision : $Id: lsql_slideshow.cgi,v 1.1 2003/03/18 18:31:20 aki Exp $
#
# Copyright (c) 2001 Gossamer Threads Inc.  All Rights Reserved.
# Redistribution in part or in whole strictly prohibited. Please
# see LICENSE file for full details.
# ==================================================================
#

use strict;
use CGI;
use vars qw/ $IN %KNOWN_FTYPES $EXT_REGEX /;

%KNOWN_FTYPES = (
        jpg         => 'image/jpeg', 
        jpeg        => 'image/jpeg', 
        gif         => 'image/gif', 
        png         => 'image/png', 
        );

$EXT_REGEX = join( "|", keys %KNOWN_FTYPES );

$IN = new CGI;

if ( $IN->param( 'do_test' ) ) {
    print $IN->header();
    run_tests();
}
elsif ( $IN->param( 'do_resize' ) ) {
    print $IN->header();
    resize_image();
}
elsif ( $IN->param( 'get_image' )) {
    display_image();
}
else {
    print $IN->header();
    display_screen(qq!
                    <p>Thank you for 
                      downloading the test script for the SlideShow Plugin 
                      for <a href="http://www.gossamer-threads.com/scripts/links-sql/index.htm">Links 
                      SQL</a>.</p>
                    <p>When you would 
                      like to start the test, please click <a href="lsql_slideshow.cgi?do_test=1">here</a>.</p>
                    !);
}


sub run_tests {
# ------------------------------------------------------------------------------
# execute the tests to find out if this system is compatible or not
#

# basic test
    my $err;
    eval {
        require Image::Magick;
    };
    if ( $@ ) {

        $err = qq!
        Cannot load Image::Magick modules. Reason: <i>($@)</i>
        
        <p>
        This plugin requires the Image::Magick perl modules which are downloadable using CPAN. Please not that these modules rely upon a compiled library called Imagemagick which can be acquired at <a href="http://www.imagemagick.org/">http://www.imagemagick.org/</a>

        !;

    }

    if ( $err ) {
        return display_screen(qq!$err!);
    }

# great! the image magic libraries are working now try a simple test.
    resize_form('');

}

sub resize_form {
# ------------------------------------------------------------------------------
# this displays a message to the user asking them to upload a file for
# the resize test
#
    my $dialog_msg = shift || '';

    my $msg = qq~
    
    <b>Great!</b> It appears your Image::Magick libraries have been installed. The next step is to make sure they're installed properly by resizing a sample image of your choice. 

<p>The <b>Image File</b> is the image you'd like to resize. The <b>Temporary File Directory</b> is a directory on your server in which this script can do its images. The script will need write access to this directory. Finally the <b>Maximum Width or Height</b> is the maximum dimensions the image you have uploaded can be. Please make this value smaller than one of the dimensions of the image or you will not see the change in the image size.

<p>In a pinch you can simply download our logo at the top of this screen and upload it to your server as a test.

$dialog_msg

<center>
<form name="form1" enctype="multipart/form-data" method="post" action="">
  <input type=hidden name=do_resize value=1>
  <table width="432" border="0" cellspacing="0" cellpadding="4" bgcolor="#bbbbbb">
    <tr>
        <td colspan=2 align=center><font face="Arial, Helvetica, sans-serif"><b>Please fill out all values of this form.</font></td>
    </tr>
    <tr> 
      <td width="50%"><font face="Arial, Helvetica, sans-serif">Image File</font></td>
      <td width="50%"> 
        <input type="file" name="file">
        <font face="Arial, Helvetica, sans-serif"></font></td>
    </tr>
    <tr> 
      <td width="50%"><font face="Arial, Helvetica, sans-serif">Temporary File 
        Directory</font></td>
      <td width="50%"><font face="Arial, Helvetica, sans-serif"> 
        <input type="text" name="tmp_dir" value="/tmp">
        </font></td>
    </tr>
    <tr> 
      <td width="50%"><font face="Arial, Helvetica, sans-serif">Maximum Width 
        or Height</font></td>
      <td width="50%"><font face="Arial, Helvetica, sans-serif"> 
        <input type="text" name="max_dim" value=200>
        </font></td>
    </tr>
    <tr align="center"> 
      <td colspan="2"> 
        <input type="submit" name="Submit" value="Attempt Resize">
      </td>
    </tr>
  </table>
</form>

    ~;
    
    display_screen( $msg );

}

sub display_image {
#'------------------------------------------------------------------------------
# dump the image to the browser
#
    my $tmp_dir = $IN->param('tmp_dir');
    my $im_type = $IN->param('img_type');
    $im_type    =~ m,($EXT_REGEX),o;
    my $efname  = "imagemagicktest.$1";

    print $IN->header( $KNOWN_FTYPES{lc($1)} );
    open IMG, "$tmp_dir/$efname" or return;
    binmode STDOUT;
    binmode IMG;
    print <IMG>;
    close IMG;

}

sub resize_image {
# ------------------------------------------------------------------------------
# attempt to resize the image
#'

    my $fhandle = $IN->param('file');
    my $tmp_dir = $IN->param('tmp_dir');
    my $err;

    $fhandle or resize_form(qq~<p><center><font color=red>A file must be uploaded for the test to work properly</font>~), return;
    $tmp_dir or resize_form(qq~<p><center><font color=red>A temporary directory must be specified</font>~), return;

# get the name of the file
    my $fname   = get_filename( "$fhandle" );    

# make sure it's of a valid type
    $fname =~ m,\.($EXT_REGEX),oi or resize_form(qq~<p><center><font color=red>A file must be of gif, jpeg, or png format</font>~), return;
    my $img_type = $1;
    my $efname = "imagemagicktest.$img_type";

# can we save to the directory?
    -w $tmp_dir or resize_form(qq~<p><center><font color=red>Cannot write to temporary directory: $tmp_dir</font>~), return;

# now just save the file...
    open IMG, ">$tmp_dir/$efname" or resize_form(qq~<p><center><font color=red>oops, IO error $!</font>~), return;
    binmode IMG;
    print IMG <$fhandle>;
    close IMG;

# load up the image to resize
    require Image::Magick;
    my $image = Image::Magick->new();
    $image or _resize_error( "Unknown error, cannot create Image::Magic Object" ), return;
    open IMG, "$tmp_dir/$efname";
    binmode IMG;
    $err = $image->Read( "$tmp_dir/$efname" );
    $err and _resize_error( $err ), return;
    close IMG;

# deal with width 
    my ( $iwidth, $iheight ) = $image->Get( 'width', 'height' );
    my $imax = $IN->param('max_dim');
    my $resize_percent = 1;

    if ( $iwidth > $imax ) {
        $resize_percent = $imax/ $iwidth;
    }

    if ( $iheight * $resize_percent > $imax) {
        $resize_percent = $imax / $iheight;
    }

    my ( $rwidth, $rheight ) = (
                                int( $iwidth*$resize_percent ), 
                                int( $iheight*$resize_percent )
                               );

    $err = $image->Scale( 
                    width => $rwidth,
                    height => $rheight
                );
    $err and _resize_error( $err ), return;

# save the thumbnail
    open IMG, ">$tmp_dir/$efname";
    $err = $image->Write( file => \*IMG, filename => "$efname" );
    $err and _resize_error( $err ), return;
    close IMG;

    my $msg = qq~
<center>
<table width="75%" border="0" cellspacing="0" cellpadding="5">
  <tr> 
    <td colspan="2" height="64"><font face="Arial, Helvetica, sans-serif">Great! 
      It appears you may have a working Image::Magick installation. If you can 
      see a resized image in the lower right corner of this panel, Image::Magick is working 
      properly.</font></td>
  </tr>
  <tr valign="top"> 
    <td width="33%"> 
      <p><font face="Arial, Helvetica, sans-serif" size="3"><b>Parameters Used</b></font></p>
      <p><font face="Arial, Helvetica, sans-serif" size="2"><b>Filename:</b><br>$efname</font></p>
      <p><font face="Arial, Helvetica, sans-serif" size="2"><b>Width x Height:</b><br>$iwidth x $iheight</font></p>
      <p><font face="Arial, Helvetica, sans-serif" size="2"><b>Thumbnail Width x Height:</b><br>$rwidth x $rheight</font></p>
      <p><font face="Arial, Helvetica, sans-serif" size="2"><b>Image Type:</b><br>$img_type</font></p>
    </td>
    <td width="67%" align=center valign=middle><font face="Arial, Helvetica, sans-serif" size="2">
        <img src="lsql_slideshow.cgi?get_image=1&tmp_dir=$tmp_dir&img_type=$img_type">
    </font></td>
  </tr>
</table>

~;

    display_screen( $msg );


}

sub _resize_error {
# -------------------------------------------------------------------
# displays and error when something goes wrong in the image magic
# process
#
    my $msg = shift;

    $msg = qq~

    Ooops! There seems to be a problem with the image resizing process. The error was: <i>($msg)</i>

    <p>
    Until this issue has been dealt with, the Image Magic test cannot go further and you will not be able to install the Image Magic plugin.

    ~;

    display_screen( $msg );
}

sub get_filename {
# -------------------------------------------------------------------
    my $fpath   = shift;
    my @path    = split /[\\\/]/, $fpath;
    return pop @path;
}

sub display_screen {
# ------------------------------------------------------------------------------
# just display the screen 
#
    my $msg = shift;

    print <<END_OF_PAGE;
<html>
<head>
<title>SlideShow plugin test</title>
<meta http-equiv="Content-Type" content="text/html; charset=iso-8859-1">
</head>

<body bgcolor="#FFFFFF" text="#000000">
<center>
  <table width="100%" border="0" cellspacing="0" cellpadding="2">
    <tr>
      <td align="center" valign="middle"> 
        <table width="75%" border="0" cellspacing="0" cellpadding="0" bgcolor="#CCCCCC" height="254">
          <tr> 
            <td width="90%" bgcolor="#000000" height="48"><img src="http://www.gossamer-threads.com/images/header.gif"></td>
          </tr>
          <tr> 
            <td width="90%" height="116"> 
              <table width="100%" border="0" cellspacing="0" cellpadding="10" height="100%">
                <tr> 
                  <td><font face="arial, helvetica"> 
                    $msg
                  </td>
                </tr>
              </table>
            </td>
          </tr>
          <tr> 
            <td width="90%" align="right" bgcolor="#000000" height="58"><font face="Arial, Helvetica, sans-serif" size="2">copyright 
              2001 gossamer threads </font></td>
          </tr>
        </table>
      </td>
    </tr>
  </table>
    <tr><td><br>&nbsp;</td></tr>
</center>
</body>
</html>

END_OF_PAGE
;
}



